(***********************************************************************)
(*                                                                     *)
(*                              Ref-image                              *)
(*                                                                     *)
(*                  Image Segmentation by Refinement                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2009-2013 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: ugen_numbering_hashtbl.mli,v 1.3 2013-07-02 16:58:46 fclement Exp $*)

(** {3 Generic numbering hash tables}

 Specialized hash tables for the case when a costless bijective hash function
 can be defined. *)

type 'a value = 'a Uindex.t;;

type ('size, 'key) t =
  {
   length : Upervasives.length;
   size : 'size;
   h : 'key -> 'key value;
   h_inv : 'key value -> 'key;
  }
;;

(** {6 Access functions} *)

val length : ('size, 'key) t -> Upervasives.length;;
val size : ('size, 'key) t -> 'size;;

val value_of_key : ('size, 'key) t -> 'key -> 'key value;;

val key_of_value : ('size, 'key) t -> 'key value -> 'key;;

(** {6 Iterators} *)

val iter : ('key -> 'key value -> unit) -> ('size, 'key) t -> unit;;

val sub_iter :
  ('key -> 'key value -> unit) ->
  ('size, 'key) t -> 'key value -> Upervasives.length -> unit
;;

val fold_left :
  ('accu -> 'key -> 'key value -> 'accu) ->
  'accu -> ('size, 'key) t -> 'accu
;;
