(***********************************************************************)
(*                                                                     *)
(*                              Ref-image                              *)
(*                                                                     *)
(*                  Image Segmentation by Refinement                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2009-2013 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: display_types.mli,v 1.20 2013-09-19 14:57:29 fclement Exp $ *)

open Segmentation_types;;

(** {6 Setting the display window} *)

type aspect =
  | Portrait
  | Landscape
;;
(** Aspect ratio of the data image. *)

type direction =
  | Horizontal
  | Vertical
;;
(** Direction of the current length. *)

type rgb_direction =
  | Rgb_Horizontal
  | Rgb_Vertical
;;
(** Direction of the Red/Green/Blue components. *)

type image_size = size
and display_image_size = size
and window_size = size
and min_window_size = window_size
and max_window_size = window_size
;;

type scaling_factor = Upositive_rational.t;;
type scaling = Unatural.t -> Unatural.t;;

type graphics_elements =
  {
   rgb_direction : rgb_direction;
   scaling_factor : scaling_factor;
   display_image_size : display_image_size;
   window_size : window_size;
  }
;;

(** {6 Displaying quantities} *)

type quantity =
  | Param
  | Data
  | Grad
;;
(** The quantities to be displayed. *)

type 'a rgb_compound =
  {
   rgb : 'a * 'a * 'a;
   compound : 'a;
  }
;;
(** [rgb] field is relative to the three Red/Green/Blue components,
 whereas [compound] field is relative to the superimposition of the three. *)

type ('a, 'b) pixmap_iter =
  (zone_number -> 'a zc_gimage -> unit) -> ('a, 'b) pimage -> unit
and 'a colormap = (color -> color) -> 'a -> color
;;

type pixmap = Generic_image.pixmap
and lower_left_corner = point
and print_pixmap = Format.formatter -> unit
and ('a, 'b) alter_pixmap =
  ('a, 'b) pixmap_iter -> 'a colormap -> 'a gimage -> ('a, 'b) pimage -> unit
and draw_pixmap = unit -> unit
;;

type pixmaps = pixmap rgb_compound
and lower_left_corners = lower_left_corner rgb_compound
and print_pixmaps = print_pixmap rgb_compound
and ('a, 'b) alter_pixmaps = ('a, 'b) alter_pixmap rgb_compound
and draw_pixmaps = draw_pixmap rgb_compound
;;

type ('a, 'b) display_quantity =
  {
   print_pixmaps : print_pixmaps;
   alter_pixmaps : ('a, 'b) alter_pixmaps;
   draw_pixmaps : draw_pixmaps;
  }
;;

type ('a, 'b) display_quantities =
  {
   dq_param : ('a, 'b) display_quantity;
   dq_data : ('a, 'b) display_quantity;
   dq_grad : ('a, 'b) display_quantity;
  }
;;
