(***********************************************************************)
(*                                                                     *)
(*                              Ref-image                              *)
(*                                                                     *)
(*                  Image Segmentation by Refinement                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2009-2013 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* Beginning of ocamlprintc.bin generated code *)

let print_component_number ppf = function
  | u_t -> Unatural_print.print_t ppf u_t
;;

let rec print_length ppf = function
  | u_l -> Upervasives_print.print_length ppf u_l

and print_width ppf = function | l -> print_length ppf l

and print_height ppf = function | l -> print_length ppf l

and print_size ppf = function
  | (w, h) ->
    Format.fprintf ppf "%,@[<1>(@,"; print_width ppf w;
    Format.fprintf ppf "%,,@ "; print_height ppf h;
    Format.fprintf ppf "%,@,)@]"
;;

let print_color ppf = function | g_c -> Graphics_print.print_color ppf g_c
;;

let print_image ppf = function
  | c_u_t -> Umatrix_print.print_t print_color ppf c_u_t
;;

let print_point ppf = function
  | (u_t, u_t0) ->
    Format.fprintf ppf "%,@[<1>(@,"; Unatural_print.print_t ppf u_t;
    Format.fprintf ppf "%,,@ "; Unatural_print.print_t ppf u_t0;
    Format.fprintf ppf "%,@,)@]"
;;

let print_gimage pp_a ppf = function
  | a_u_t -> Umatrix_graph_print.print_t pp_a ppf a_u_t
;;

let print_indicator pp_a ppf = function | a -> pp_a ppf a
;;

let print_zone_cutting_candidate pp_a ppf = function
  | { Segmentation_types.indicator = a_i; Segmentation_types.cutting = u_d; } ->
     Format.fprintf ppf "%,@[<1>{@,";
     Format.fprintf ppf "%,@[<1>indicator =@ "; print_indicator pp_a ppf a_i;
     Format.fprintf ppf "%,;@]@ "; Format.fprintf ppf "%,@[<1>cutting =@ ";
     Umatrix_nonempty_set_print.print_dichotomy ppf u_d;
     Format.fprintf ppf "%,;@]@ "; Format.fprintf ppf "%,@,}@]"
;;

let print_best_zone_cutting pp_a ppf = function
  | Segmentation_types.No_more_zone_cutting ->
    Format.fprintf ppf "%s" "No_more_zone_cutting"
  | Segmentation_types.Best_zone_cutting a_z ->
    Format.fprintf ppf "@[<1>%s@ " "Best_zone_cutting";
    print_zone_cutting_candidate pp_a ppf a_z; Format.fprintf ppf "%,@]"
;;

let print_cost pp_a ppf = function | a -> pp_a ppf a
;;

let print_zone_constants pp_a ppf = function
  | { Segmentation_types.zc_param = a; Segmentation_types.zc_cost = a_c; } ->
    Format.fprintf ppf "%,@[<1>{@,";
    Format.fprintf ppf "%,@[<1>zc_param =@ "; pp_a ppf a;
    Format.fprintf ppf "%,;@]@ "; Format.fprintf ppf "%,@[<1>zc_cost =@ ";
    print_cost pp_a ppf a_c; Format.fprintf ppf "%,;@]@ ";
    Format.fprintf ppf "%,@,}@]"
;;

let print_zc_gimage pp_a ppf = function
  | a_z_g -> print_gimage (print_zone_constants pp_a) ppf a_z_g
;;

let print_zone pp_a ppf = function
  | a_z_a_b_u_b ->
    Umatrix_structural_mapping_print.print_block (print_zone_constants pp_a)
     (print_best_zone_cutting pp_a) ppf a_z_a_b_u_b
;;

let print_zone_number ppf = function
  | u_b -> Umatrix_structural_mapping_print.print_block_number ppf u_b
;;

let print_bounds ppf = function
  | { Segmentation_types.ll = p; Segmentation_types.ur = p0; } ->
    Format.fprintf ppf "%,@[<1>{@,"; Format.fprintf ppf "%,@[<1>ll =@ ";
    print_point ppf p; Format.fprintf ppf "%,;@]@ ";
    Format.fprintf ppf "%,@[<1>ur =@ "; print_point ppf p0;
    Format.fprintf ppf "%,;@]@ "; Format.fprintf ppf "%,@,}@]"
;;

let print_pimage pp_a pp_b ppf = function
  | a_z_a_b_b_u_t ->
    Umatrix_structural_mapping_print.print_t (print_zone_constants pp_a)
     (print_best_zone_cutting pp_a) pp_b ppf a_z_a_b_b_u_t
;;

let print_cimage pp_a pp_b ppf = function
  | (a_c, a_b_p) ->
    Format.fprintf ppf "%,@[<1>(@,"; print_cost pp_a ppf a_c;
    Format.fprintf ppf "%,,@ "; print_pimage pp_a pp_b ppf a_b_p;
    Format.fprintf ppf "%,@,)@]"
;;

let print_zonation pp_a pp_b ppf = function
  | {
      Segmentation_types.component = c;
      Segmentation_types.other_components = c_l;
      Segmentation_types.gimage_data = a_g;
      Segmentation_types.cimage = a_b_c;
    } ->
    Format.fprintf ppf "%,@[<1>{@,";
    Format.fprintf ppf "%,@[<1>component =@ "; print_component_number ppf c;
    Format.fprintf ppf "%,;@]@ ";
    Format.fprintf ppf "%,@[<1>other_components =@ ";
    Lib_print.print_List print_component_number ppf c_l;
    Format.fprintf ppf "%,;@]@ ";
    Format.fprintf ppf "%,@[<1>gimage_data =@ "; print_gimage pp_a ppf a_g;
    Format.fprintf ppf "%,;@]@ "; Format.fprintf ppf "%,@[<1>cimage =@ ";
    print_cimage pp_a pp_b ppf a_b_c; Format.fprintf ppf "%,;@]@ ";
    Format.fprintf ppf "%,@,}@]"
;;

let print_output_status ppf = function
  | Segmentation_types.Max_iteration h_s ->
    Format.fprintf ppf "@[<1>%s@ " "Max_iteration";
    History_types_print.print_state_number ppf h_s; Format.fprintf ppf "%,@]"
  | Segmentation_types.Min_scalar_indicator f ->
    Format.fprintf ppf "@[<1>%s@ " "Min_scalar_indicator";
    Lib_print.print_quoted_float ppf f; Format.fprintf ppf "%,@]"
  | Segmentation_types.Min_scalar_relative_error f ->
    Format.fprintf ppf "@[<1>%s@ " "Min_scalar_relative_error";
    Lib_print.print_quoted_float ppf f; Format.fprintf ppf "%,@]"
  | Segmentation_types.No_more_cuttings ->
    Format.fprintf ppf "%s" "No_more_cuttings"
;;

let print_number_of_cuts ppf = function
  | u_t -> Unatural_print.print_t ppf u_t
;;

let print_basic_cutting_family ppf = function
  | Segmentation_types.Horizontal -> Format.fprintf ppf "%s" "Horizontal"
  | Segmentation_types.Vertical -> Format.fprintf ppf "%s" "Vertical"
;;

let print_cutting_strategy ppf = function
  | Segmentation_types.Best_in_a_family (n, b_u_l) ->
    Format.fprintf ppf "@[<1>%s@ " "Best_in_a_family";
    Format.fprintf ppf "%,@[<1>(@,"; print_number_of_cuts ppf n;
    Format.fprintf ppf "%,,@ ";
    Ulist1_print.print_list1 print_basic_cutting_family ppf b_u_l;
    Format.fprintf ppf "%,@,)@]"; Format.fprintf ppf "%,@]"
  | Segmentation_types.Overall_best -> Format.fprintf ppf "%s" "Overall_best"
;;

let print_cutting pp_a ppf = function
  | f_c_f_a_g_f_a_z_a_b ->
    Lib_print.print_fun print_cutting_strategy
     (Lib_print.print_fun (print_gimage pp_a)
       (Lib_print.print_fun (print_zc_gimage pp_a)
         (print_best_zone_cutting pp_a)
       )
     ) ppf f_c_f_a_g_f_a_z_a_b
;;

let print_component_cutting_candidate pp_a ppf = function
  | {
      Segmentation_types.zone_number = z;
      Segmentation_types.zone_cutting_candidate = a_z;
    } ->
    Format.fprintf ppf "%,@[<1>{@,";
    Format.fprintf ppf "%,@[<1>zone_number =@ "; print_zone_number ppf z;
    Format.fprintf ppf "%,;@]@ ";
    Format.fprintf ppf "%,@[<1>zone_cutting_candidate =@ ";
    print_zone_cutting_candidate pp_a ppf a_z; Format.fprintf ppf "%,;@]@ ";
    Format.fprintf ppf "%,@,}@]"
;;

let print_best_component_cutting pp_a ppf = function
  | Segmentation_types.No_more_component_cutting ->
    Format.fprintf ppf "%s" "No_more_component_cutting"
  | Segmentation_types.Best_component_cutting a_c ->
    Format.fprintf ppf "@[<1>%s@ " "Best_component_cutting";
    print_component_cutting_candidate pp_a ppf a_c; Format.fprintf ppf "%,@]"
;;

let print_global_cutting_candidate pp_a ppf = function
  | {
      Segmentation_types.component_number = c;
      Segmentation_types.component_cutting_candidate = a_c;
    } ->
    Format.fprintf ppf "%,@[<1>{@,";
    Format.fprintf ppf "%,@[<1>component_number =@ ";
    print_component_number ppf c; Format.fprintf ppf "%,;@]@ ";
    Format.fprintf ppf "%,@[<1>component_cutting_candidate =@ ";
    print_component_cutting_candidate pp_a ppf a_c;
    Format.fprintf ppf "%,;@]@ "; Format.fprintf ppf "%,@,}@]"
;;

let print_best_global_cutting pp_a ppf = function
  | Segmentation_types.No_more_global_cutting ->
    Format.fprintf ppf "%s" "No_more_global_cutting"
  | Segmentation_types.Best_global_cutting a_g ->
    Format.fprintf ppf "@[<1>%s@ " "Best_global_cutting";
    print_global_cutting_candidate pp_a ppf a_g; Format.fprintf ppf "%,@]"
;;

(* End of ocamlprintc.bin generated code *)
