(***********************************************************************)
(*                                                                     *)
(*                              Ref-image                              *)
(*                                                                     *)
(*                  Image Segmentation by Refinement                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2009-2013 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: zonation.ml,v 1.70 2013-12-02 15:45:01 fclement Exp $ *)

module Nat = Unatural;;
module Dgraph = Udynamic_graph;;
module Ddgraph = Udelayed_dgraph;;

module Set = Umatrix_nonempty_set;;
module Mapping = Umatrix_graph;;

open Segmentation_types;;
open Image;;

(*
let gen_segmentation_digest =
  Utype_digest.gen_unary_digest Segmentation_digest.md5
;;
let digest = gen_segmentation_digest "zonation";;
*)

let gimage_of_zonation z = gimage_of_cimage z.cimage;;

let make_trivial_zonation
    cut mean cost_sub cost gdata channel other_channell =
  {
   component = channel;
   other_components = other_channell;
   gimage_data = gdata;
   cimage = cimage_of_gimage cut mean cost_sub cost gdata;
  }
;;

let cartesian_cuts ijmin ijmax dom p nct =
  (* nij = size of the box *)
  let nij = Nat.succ (Nat.sub ijmax ijmin) in
  (* nct is the maximum number of cuts,
     therefore nct + 1 is the maximum number of intervals.
     The value -1 means all possible cuts. *)
  let nint = if nct = Nat._0_ then nij else Nat.succ nct in
  let ijl = Nat.psplit nij nint in
  let rec loop accu = function
    | ([], _) -> assert false
    | (k :: kl, ijc) when ijc = ijmax ->
        if kl = [] then accu else
        loop accu (kl, Nat.sub ijc k)
    | (k :: kl, ijc) ->
        let dicho = Set.dichotomy (p ijc) dom in
        let accu = dicho :: accu in
        if kl = [] then accu else
        loop accu (kl, Nat.sub ijc k) in
  loop [] (ijl, ijmax)
;;

let horizontal_cuts { ll = (imin, _); ur = (imax, _); } dom nct =
  let pi ic idx _uidx =
    let (i, _) = Set.element dom idx in
    i <= ic in
  cartesian_cuts imin imax dom pi nct
;;

let vertical_cuts { ll = (_, jmin); ur = (_, jmax); } dom nct =
  let pj jc idx _uidx =
    let (_, j) = Set.element dom idx in
    j <= jc in
  cartesian_cuts jmin jmax dom pj nct
;;

let cuttings_from_family bds dom nct = function
  | Horizontal -> horizontal_cuts bds dom nct
  | Vertical -> vertical_cuts bds dom nct
;;

let indicator_of_zcc zcc = zcc.indicator;;
let indicator_of_ccc ccc = indicator_of_zcc ccc.zone_cutting_candidate;;
let indicator_of_gcc gcc = indicator_of_ccc gcc.component_cutting_candidate;;

let bzc_of indic dicho =
  let zcc = { indicator = indic; cutting = dicho; } in
  Best_zone_cutting zcc
;;

let bcc_of zone_nb zcc =
  let ccc = { zone_number = zone_nb; zone_cutting_candidate = zcc; } in
  Best_component_cutting ccc
;;

let bgc_of comp_nb ccc =
  let bgc =
    { component_number = comp_nb;
      component_cutting_candidate = ccc; } in
  Best_global_cutting bgc
;;

let better_cutting accu zone_nb zone =
  match (accu, zone.Dgraph.image) with
  | (_, Ddgraph.Not_yet_computed) -> assert false
  | (_, Ddgraph.Image No_more_zone_cutting) -> accu
  | (No_more_component_cutting,
     Ddgraph.Image (Best_zone_cutting zcc)) -> bcc_of zone_nb zcc
  | (Best_component_cutting best_ccc,
     Ddgraph.Image (Best_zone_cutting zcc)) ->
      if indicator_of_zcc zcc <= indicator_of_ccc best_ccc then accu else
      bcc_of zone_nb zcc
;;

let same z1 z2 =
  same_gimage z1.gimage_data z2.gimage_data &&
  same_cimage z1.cimage z2.cimage
;;
