(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2004-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: uchar.mli,v 1.7 2014/01/10 15:28:04 fclement Exp $ *)

(** {3 More character operations} *)

exception Unextended_char of char;;
exception Digram of char * char;;

val iso_latin1_to_us_ascii : char -> char;;
(** Return the non accented version of characters of the ISO Latin-1 (8859-1)
   character set.
   
   Raise [Unextended_char] for non alphabetic non US ASCII characters.
   Raise [Digram] with the two characters for digrams.
   E.g. [us_ascii_transcription_of_iso_latin1 '\198'] raises
   [Digram ('A', 'E')]. *)

val is_us_ascii : char -> bool;;
val is_alphabetic : char -> bool;;
val is_numeric : char -> bool;;
val is_alphanumeric : char -> bool;;
val is_space : char -> bool;;
val is_punctuation : char -> bool;;
val is_mathematical : char -> bool;;
(** Indicator functions for common subsets of the ISO Latin-1 (8859-1)
   character set. *)
