(***********************************************************************)
(*                                                                     *)
(*                              Ref-image                              *)
(*                                                                     *)
(*                  Image Segmentation by Refinement                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2009-2013 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: uset.mli,v 1.1 2013/06/04 17:11:41 fclement Exp $ *)

type depth =
  | Universe_index
  | Set_index of depth

and 'a support_index = 'a Uuniverse.index

and 'a index =
  {
   depth : depth;
   value : 'a support_index Uuniverse.index;
  }
;;

type 'a support =
  | Universe of 'a Uuniverse.t
  | Set of 'a set

and 'a set =
  | Empty of 'a support
  | Nonempty of 'a nonempty

and 'a nonempty =
  {
   support : 'a support;
   indices : 'a support_index Uuniverse.t;
  }
;;

type 'a t = 'a set;;

val empty : 'a support -> 'a set;;

val singleton : 'a support -> 'a support_index -> 'a set;;

val mem : 'a set -> 'a index -> bool;;

val add_to_nonempty : 'a nonempty -> 'a support_index -> unit;;

val add : 'a set -> 'a index -> 'a set;;

val choice : 'a set -> 'a index;;

val indices : 'a set -> 'a index array;;

val element : 'a set -> 'a index -> 'a;;

val elements : 'a set -> 'a array;;

val cardinal : 'a set -> Unatural.t;;

val valid_index : 'a set -> 'a index -> bool;;
val check_index : string -> 'a set -> 'a index -> unit;;

val union : 'a set -> 'a set -> 'a set;;

val fold_left : ('a -> 'b index -> 'a) -> 'a -> 'b set -> 'a;;

val iter : ('a index -> unit) -> 'a set -> unit;;

val map_indices : ('a index -> 'b) -> 'a set -> 'b array;;

val map :
  ('a index -> 'a support_index) -> 'a set -> 'b support -> 'b set
;;

val find : ('a index -> bool) -> 'a set -> 'a index;;

val find_all :
  ('a index -> bool) -> 'a set -> 'a index list
;;

val filter :
  ('a index -> bool) -> 'a set -> 'a index list
;;

(*
val flatten : 'a set set -> 'a set;;
*)

val to_list : 'a set -> 'a index list;;

val of_array : 'a support -> 'a support_index array -> 'a set;;
val of_list : 'a support -> 'a support_index list -> 'a set;;

(** {6 Quotient set} *)

(*
type 'a quotient_set = 'a set list;;

val quotient_set :
  'a set -> ('a index -> Unatural.t) -> 'a quotient_set
;;
*)

val dichotomy : ('a index -> bool) -> 'a set -> 'a set * 'a set;;
